#version 150 compatibility // -*- c++ -*-
#extension GL_EXT_geometry_shader4 : enable

#include "clip.glsl"
#line 8

layout(triangles) in;
layout(triangle_strip, max_vertices = 5) out;

in vec4 lastVertex[3];

/** If true, the input is a quadrilateral.  Otherwise the input is a triangle. */
uniform bool quadSource;
uniform mat4 projectionMatrix;
uniform vec3 n;

void main() {

    vec3 v[5];
    for (int i = 0; i < 3; ++i) {
        v[i] = gl_PositionIn[i].xyz;
    }

    int numVerts;


    if (quadSource) {
        v[3] = lastVertex[0].xyz;
        numVerts = clip4(n, v[0], v[1], v[2], v[3], v[4]);
    } else {
        numVerts = clip3(n, v[0], v[1], v[2], v[3]);
    }


    if (numVerts > 0) {
        // Polygon winding order:
        //   0___2     0___3      0___4
        //   |  /      |  /|      |  /|\ 
        //   | /       | / |      | / | 3
        //   |/        |/__|      |/__|/ 
        //   1         1   2      1   2    
        //
        // Corresponding tristrip winding order:
        //
        //   012       0132       01423
        //

#       define EMIT(i) \
            gl_Position = projectionMatrix * g3d_WorldToCameraMatrix * vec4(v[i], 1.0);\
            EmitVertex()

        EMIT(0);
        EMIT(1);

        if (numVerts == 4) {
            EMIT(3);
        } else if (numVerts == 5) {
            EMIT(4);
        }

        EMIT(2);
        
        if (numVerts == 5) {
            EMIT(3);
        }
    }
}
